
#Region "Comments"

'Name: Hemesh
'Date: 20.03.2008
'Description: Base class implementing Partial IEditableObject and IDataObject

#End Region



#Region "Imports"

Imports System.ComponentModel
Imports System.Reflection
Imports System.Xml
Imports System.Drawing
Imports System.Text
Imports System.Runtime.Serialization
Imports System.Runtime.Serialization.Formatters.Binary
Imports System.IO
Imports System.Collections.Generic

 
#End Region




<Serializable()> _
Public Class BDOBase
    Implements IDataObject
    Implements IEditableObject
    'Implements ICloneable

    Public Const C_NEW_ID As Integer = -1

#Region "Variables"

    'Integer
    Private intID As Integer            'Id of the item.

    ''Boolean
    Private blnIsDirty As Boolean
    Private blnIsEditing As Boolean
    Private blnIsEditingDirty As Boolean  'Variable to check the Data Object is in Edit Mode or, Not.

    Private blnIsSelected As Boolean
    Private blnIsDeleted As Boolean
    Private blnIsSystem As Boolean
    Private blnIsPartofCollection As Boolean
    Private blnInTrans As Boolean
    Private objBackUpObject As BDOBase

    Private strUniqueIdentifier As String


    'Enums
    Public Enum EBDOBase
        ID
        IsSelected
        IsDeleted
        IsSystem
        IsPartOfCollection
        IsDirty
        IsNew
        IsEditingDirty
        IsEditing
        UniqueIdentifier
        UniqueID
    End Enum


#End Region


#Region "Events"

    Public Event PropertychangedEvent(ByVal sender As Object, ByVal e As PropertyChangedEventArgs) Implements IDataObject.PropertychangedEvent
    Public Event RemoveMe(ByVal o As Object, ByVal e As EventArgs) Implements IDataObject.RemoveMe

#End Region


#Region "Property"


    ''' <summary>
    ''' Used just in this class for Begin and Cancel edit
    ''' we keep the backup of the current state in the object here
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Property BackUpObject() As BDOBase
        Get
            Return objBackUpObject
        End Get
        Set(ByVal value As BDOBase)
            objBackUpObject = value
        End Set
    End Property




    Public Property IsSelected() As Boolean Implements IDataObject.IsSelected
        Get
            Return blnIsSelected
        End Get
        Set(ByVal value As Boolean)
            If blnIsSelected <> value Then
                blnIsSelected = value
                OnPropertyChanged(EBDOBase.IsSelected)
            End If
        End Set
    End Property



    Public Property IsSystem() As Boolean Implements IDataObject.IsSystem
        Get
            Return blnIsSystem
        End Get
        Set(ByVal value As Boolean)
            If blnIsSystem <> value Then
                blnIsSystem = value
                OnPropertyChanged(EBDOBase.IsSystem)
            End If
        End Set
    End Property

    Public Property IsPartOfCollection() As Boolean Implements IDataObject.IsPartOfCollection
        Get
            Return blnIsPartofCollection
        End Get
        Set(ByVal value As Boolean)
            If blnIsPartofCollection <> value Then
                blnIsPartofCollection = value
                'HS Since this is for internal use so we dont want oject to gte dirty here
                'OnPropertyChanged(EBDOBase.IsPartOfCollection)
            End If
        End Set
    End Property

    Protected Property InTrans() As Boolean
        Get
            Return blnInTrans
        End Get
        Set(ByVal value As Boolean)
            If blnInTrans <> value Then
                blnInTrans = value
                'HS Since this is for internal use so we dont want oject to gte dirty here
                'OnPropertyChanged(EBDOBase.blnInTrans)
            End If
        End Set
    End Property


    Public ReadOnly Property UniqueIdentifier() As String Implements IDataObject.UniqueIdentifier
        Get
            Return strUniqueIdentifier
        End Get
    End Property


    Public ReadOnly Property UniqueID() As String Implements IDataObject.UniqueID
        Get
            If IsNew Then
                Return UniqueIdentifier
            Else
                Return ID
            End If
        End Get
    End Property


    Public Sub SetUniqueIdentifier() Implements IDataObject.SetUniqueIdentifier
        strUniqueIdentifier = Now.Ticks.ToString
    End Sub


#End Region


#Region "Constrcutor"

    Public Sub New()
        ID = C_NEW_ID
        Me.SetUniqueIdentifier()
    End Sub

#End Region


#Region "IDataObject"



    Public Property ID() As Integer Implements IDataObject.ID
        Get
            Return intID
        End Get
        Set(ByVal Value As Integer)
            If intID <> Value Then
                intID = Value
                OnPropertyChanged(EBDOBase.ID)
            End If
        End Set
    End Property


    Public Overridable Property IsDirty() As Boolean Implements IDataObject.IsDirty
        Get
            Return blnIsDirty 'OrElse GetIsObjectDirty()
        End Get
        Set(ByVal Value As Boolean)
            If blnIsDirty <> Value Then
                blnIsDirty = Value
                If Not Value Then
                    Me.IsEditingDirty = False
                    'ClearObjectDirty(Me.GetType)
                End If
            End If
        End Set
    End Property



    'Private Sub ClearObjectDirty(ByVal currType As Type)
    '    Dim Flds() As FieldInfo
    '    Dim FldInfo As FieldInfo
    '    Dim ty As Type
    '    Dim tempobj As IDataObject

    '    Flds = currType.GetFields(BindingFlags.DeclaredOnly _
    '                        Or BindingFlags.Instance _
    '                        Or BindingFlags.NonPublic _
    '                        Or BindingFlags.Public)

    '    For Each FldInfo In Flds
    '        ty = FldInfo.FieldType.GetInterface(GetType(IDataObject).Name, True)

    '        If ty IsNot Nothing Then
    '            tempobj = FldInfo.GetValue(Me)
    '            If tempobj IsNot Nothing Then
    '                tempobj.IsDirty = False
    '            End If
    '        End If
    '    Next

    '    If currType.BaseType IsNot Nothing AndAlso Not currType.BaseType.Name = GetType(Object).Name Then
    '        ClearObjectDirty(currType.BaseType)
    '    End If

    'End Sub


    'Private Function GetIsObjectDirty() As Boolean
    '    Return GetIsObjectDirty(Me.GetType)
    'End Function
    'Private Function GetIsObjectDirty(ByVal currType As Type) As Boolean
    '    Dim Flds() As FieldInfo
    '    Dim FldInfo As FieldInfo
    '    Dim ty As Type
    '    Dim tempobj As IDataObject

    '    Flds = currType.GetFields(BindingFlags.DeclaredOnly _
    '                        Or BindingFlags.Instance _
    '                        Or BindingFlags.NonPublic _
    '                        Or BindingFlags.Public)

    '    For Each FldInfo In Flds
    '        ty = FldInfo.FieldType.GetInterface(GetType(IDataObject).Name, True)

    '        If ty IsNot Nothing Then
    '            tempobj = FldInfo.GetValue(Me)
    '            If tempobj IsNot Nothing Then
    '                If tempobj.IsDirty Then
    '                    Return True
    '                End If
    '            End If
    '        End If
    '    Next

    '    If currType.BaseType IsNot Nothing AndAlso Not currType.BaseType.Name = GetType(Object).Name Then
    '        Return GetIsObjectDirty(currType.BaseType)
    '    End If

    'End Function



    Public Overridable ReadOnly Property IsNew() As Boolean Implements IDataObject.IsNew
        Get
            Return ID = C_NEW_ID
        End Get
    End Property

    Public Property IsDeleted() As Boolean Implements IDataObject.IsDeleted
        Get
            Return blnIsDeleted
        End Get
        Set(ByVal value As Boolean)
            If blnIsDeleted <> value Then
                blnIsDeleted = value
                OnPropertyChanged(EBDOBase.IsDeleted)
            End If
        End Set
    End Property


#End Region


#Region "IEditableObject"


    Protected Property IsEditingDirty() As Boolean
        Get
            Return blnIsEditingDirty
        End Get
        Set(ByVal Value As Boolean)
            blnIsEditingDirty = Value
        End Set
    End Property


    Protected Property IsEditing() As Boolean
        Get
            Return blnIsEditing
        End Get
        Set(ByVal Value As Boolean)
            blnIsEditing = Value
        End Set
    End Property


    Public Sub BeginEdit() Implements System.ComponentModel.IEditableObject.BeginEdit
        If IsEditing Then Exit Sub
        'BackUpObject = Clone() 'Make the Clone of the Origninal Object
        IsEditing = True
    End Sub


    Public Sub CancelEdit() Implements System.ComponentModel.IEditableObject.CancelEdit
        If Not IsEditing Then Exit Sub

        IsEditing = False

        If Not Me.IsEditingDirty Then
            If Me.IsNew Then Me.OnRemoveMe(EventArgs.Empty)
        Else
            'BackUpObject.CopyTo(Me) 'Copy the Backup object into the original object
        End If
        IsEditingDirty = False
    End Sub


    Public Sub EndEdit() Implements System.ComponentModel.IEditableObject.EndEdit
        If Not IsEditing Then Exit Sub

        IsEditing = False

        If IsEditingDirty Then
            OnPropertyChanged()
            IsEditingDirty = False
        End If
    End Sub


#End Region


#Region "Methods"


    Protected Overridable Sub OnPropertyChanged(ByVal PropertyName As [Enum])
        Dim strPropName As String
        If Not IsEditing Or Not PropertyName Is Nothing Then
            'Try
            '    strPropName = Me.GetType.GetProperty(PropertyName.ToString).Name
            'Catch ex As AmbiguousMatchException
            '    ' if same name property found in base and child, use the base property
            '    strPropName = Me.GetType.BaseType.GetProperty(PropertyName.ToString).Name
            'End Try
            strPropName = PropertyName.ToString
            OnPropertyChanged(strPropName)
            IsDirty = True
        Else
            IsEditingDirty = True
        End If
    End Sub

    'please do not call this method from the set property by doing enum.tostring
    ' this method is only to be used when child do's property changes and base do
    'is listening to that change. in that case the base do gets only the propname
    'and not the enum
    Protected Sub OnPropertyChanged(Optional ByVal PropertyName As String = Nothing)
        RaiseEvent PropertychangedEvent(Me, New PropertyChangedEventArgs(PropertyName))
    End Sub


    Protected Sub OnRemoveMe(ByVal e As EventArgs)
        RaiseEvent RemoveMe(Me, e)
    End Sub


    Public Overridable Function GetEnumValue(ByVal enm As System.Enum) As Object Implements IDataObject.GetEnumValue
        Dim value As Object
        value = Me.GetType.GetProperty(enm.ToString).GetValue(Me, Nothing)
        Return value
    End Function


    Public Overloads Shared Function GetBasePropertyName(ByVal enmProperty As [Enum]) As String
        Try
            Return enmProperty.ToString
        Catch ex As Exception
            Throw ex
        End Try
    End Function


    Public Overloads Shared Function GetPropertyName(ByVal enmProperty As EBDOBase) As String
        Try
            Return GetBasePropertyName(enmProperty)
        Catch ex As Exception
            Throw ex
        End Try
    End Function



    ''#Region "DO To XML"


    ''    Private Function CreateXMLWriter(ByRef strBuilder As StringBuilder) As XmlWriter
    ''        Dim xmlWrt As XmlWriter
    ''        Dim xmlSettings As XmlWriterSettings

    ''        strBuilder = New StringBuilder
    ''        xmlSettings = New XmlWriterSettings

    ''        xmlSettings.OmitXmlDeclaration = True
    ''        xmlSettings.CheckCharacters = True
    ''        xmlSettings.CloseOutput = True
    ''        xmlSettings.Indent = True
    ''        'This is needed as we write another XML in between tags
    ''        xmlSettings.ConformanceLevel = ConformanceLevel.Fragment

    ''        xmlWrt = XmlWriter.Create(strBuilder, xmlSettings)

    ''        Return xmlWrt

    ''    End Function


    ''    Public Overridable Function DOToXML(Optional ByVal xmlwr As XmlWriter = Nothing, _
    ''                                        Optional ByVal strFldName As String = Nothing) As String Implements IDataObject.DOToXML
    ''        Dim sys() As FieldInfo
    ''        Dim str As String
    ''        Dim xmlWrt As XmlWriter
    ''        Dim strBuilder As StringBuilder
    ''        Dim T As Type
    ''        Dim strName As String

    ''        Try

    ''            T = Me.GetType
    ''            sys = T.GetFields(BindingFlags.DeclaredOnly _
    ''                            Or BindingFlags.Instance _
    ''                            Or BindingFlags.NonPublic _
    ''                            Or BindingFlags.Public)

    ''            'For the root tag we want to write the class name
    ''            'where as for the properties of type DO in the Parent class,
    ''            ' we want to write the property name as in the class in the XML file
    ''            If xmlwr Is Nothing Then
    ''                xmlWrt = CreateXMLWriter(strBuilder)
    ''                strName = T.Name
    ''            Else
    ''                xmlWrt = xmlwr
    ''                strName = strFldName
    ''            End If

    ''            'Writes the Start Tag of the Clas name
    ''            xmlWrt.WriteStartElement(strName)

    ''            If sys.Length > 0 Then
    ''                For i As Integer = 0 To sys.Length - 1
    ''                    If sys(i).GetCustomAttributes(GetType(NonXMLSerializableAttribute), False).Length = 0 Then

    ''                        DOToXMLPrimitiveTypes(xmlWrt, sys(i), str)
    ''                        'Writes the Start, End and Value of the Property
    ''                        If Not IsBlank(str) Then xmlWrt.WriteElementString(sys(i).Name, str)
    ''                        str = ""

    ''                    End If
    ''                Next

    ''                If T.BaseType IsNot Nothing AndAlso Not T.BaseType.Name = GetType(Object).Name Then
    ''                    Me.DOToXMLBase(T.BaseType, xmlWrt)
    ''                End If

    ''            End If

    ''            'Writes the End tag of the Class name
    ''            xmlWrt.WriteEndElement()

    ''            If xmlwr Is Nothing Then xmlWrt.Close()

    ''        Catch ex As Exception
    ''            Throw ex
    ''        End Try

    ''        If Not strBuilder Is Nothing Then
    ''            Return strBuilder.ToString
    ''        End If

    ''    End Function


    ''    Public Overridable Sub DOToXMLBase(ByVal T As Type, ByVal xmlwrt As XmlWriter)
    ''        Dim sys() As FieldInfo
    ''        Dim str As String

    ''        Try

    ''            sys = T.GetFields(BindingFlags.DeclaredOnly _
    ''                            Or BindingFlags.Instance _
    ''                            Or BindingFlags.NonPublic _
    ''                            Or BindingFlags.Public)

    ''            If sys.Length > 0 Then
    ''                For i As Integer = 0 To sys.Length - 1
    ''                    If sys(i).GetCustomAttributes(GetType(NonXMLSerializableAttribute), False).Length = 0 Then

    ''                        DOToXMLPrimitiveTypes(xmlwrt, sys(i), str)
    ''                        'Writes the Start, End and Value of the Property
    ''                        If Not IsBlank(str) Then xmlwrt.WriteElementString(sys(i).Name, str)
    ''                        str = ""

    ''                    End If
    ''                Next

    ''                If T.BaseType IsNot Nothing AndAlso Not T.BaseType.Name = GetType(Object).Name Then
    ''                    Me.DOToXMLBase(T.BaseType, xmlwrt)
    ''                End If

    ''            End If

    ''        Catch ex As Exception
    ''            Throw ex
    ''        End Try


    ''    End Sub


    ''    Private Sub DOToXMLPrimitiveTypes(ByVal xmlWrt As XmlWriter, ByVal sys As FieldInfo, ByRef str As String)
    ''        Select Case sys.FieldType.Name
    ''            Case GetType(String).Name
    ''                str = ObjSQLConverter.DBToString(sys.GetValue(Me))

    ''            Case GetType(DateTime).Name
    ''                str = ObjSQLConverter.DBToString(sys.GetValue(Me))

    ''            Case GetType(Color).Name
    ''                str &= DirectCast(sys.GetValue(Me), Color).ToArgb

    ''            Case GetType(Decimal).Name
    ''                str = ObjSQLConverter.DBToString(sys.GetValue(Me))

    ''            Case Else
    ''                If sys.FieldType.IsPrimitive Then
    ''                    str = ObjSQLConverter.DBToString(sys.GetValue(Me))

    ''                ElseIf sys.FieldType.IsArray Then
    ''                    If Not sys.GetValue(Me) Is Nothing Then
    ''                        For i As Integer = 0 To sys.GetValue(Me).length - 1
    ''                            str &= ObjSQLConverter.DBToString(sys.GetValue(Me)(i)) & ","
    ''                        Next
    ''                        str = str.Remove(str.LastIndexOf(","), 1)
    ''                    End If

    ''                ElseIf sys.FieldType.IsEnum Then
    ''                    str = sys.GetValue(Me)

    ''                Else
    ''                    DOToXMLNonPrimitiveTypes(xmlWrt, sys, str)
    ''                End If

    ''        End Select
    ''    End Sub


    ''    Private Sub DOToXMLNonPrimitiveTypes(ByVal xmlWrt As XmlWriter, ByVal sys As FieldInfo, _
    ''                                        ByRef str As String)
    ''        Dim ty As Type
    ''        Dim obj As BDOBase
    ''        Dim formatter As BinaryFormatter
    ''        Dim strm As New MemoryStream

    ''        ty = sys.FieldType.GetInterface(GetType(MSPL.DO.IDataObject).Name, True)

    ''        If ty IsNot Nothing Then
    ''            obj = sys.GetValue(Me)
    ''            'we need to pass here the property name of the object, as we want that it writes the name
    ''            ' in the XML file and not the type as it would have happened if we do just
    ''            'xmlWrt.WriteStartElement(T.Name)
    ''            'ex. we want this:
    ''            '<DOGeneralPreference>
    ''            '- <objAcc>
    ''            '  </objAcc>

    ''            'and not this:
    ''            '<DOGeneralPreference>
    ''            '- <DOAccountPreference>
    ''            '  </DOAccountPreference>

    ''            'So that we can find it easily from the list of the class's property

    ''            If obj IsNot Nothing Then obj.DOToXML(xmlWrt, sys.Name)
    ''        Else
    ''            'means that its not our Object, so we simple Serialize it
    ''            Dim arr() As Attribute = sys.FieldType.GetCustomAttributes(GetType(SerializableAttribute), True) 'GetInterface(GetType(ISerializable).Name, True)
    ''            If arr IsNot Nothing AndAlso Not arr.Length = 0 Then
    ''                formatter = New BinaryFormatter
    ''                formatter.Serialize(strm, sys.GetValue(Me))
    ''                str = ByteArrayToString(strm.ToArray)
    ''            End If
    ''        End If

    ''    End Sub


    ''#End Region



    ''#Region "XML To DO"

    ''    Private blnIsAlienObject As Boolean 'This flags that the next object to be poplated is not our object. we get the value after only a reader.read, so we cannot populate it directly without knowing the object type

    ''    Private Property IsAlienObject() As Boolean
    ''        Get
    ''            Return blnIsAlienObject
    ''        End Get
    ''        Set(ByVal value As Boolean)
    ''            blnIsAlienObject = value
    ''        End Set
    ''    End Property

    ''    Protected Overridable Overloads Function XMLToDO(ByVal xmlReader As System.Xml.XmlTextReader) As Hashtable Implements IDataObject.XMLToDO
    ''        Return XMLToDO(xmlReader, Nothing)
    ''    End Function


    ''    Public Overloads Sub XMLToDO(ByVal XMLContent As String)
    ''        Dim xmlreader As XmlReader
    ''        If XMLContent Is Nothing OrElse XMLContent.Length = 0 Then Exit Sub

    ''        Try
    ''            xmlreader = XMLUtility.GetXmlReader(XMLContent)
    ''            XMLToDO(xmlreader)
    ''        Catch ex As Exception
    ''            Throw ex
    ''        Finally
    ''            If Not xmlreader Is Nothing Then xmlreader.Close()
    ''        End Try
    ''    End Sub


    ''    ''' <summary>
    ''    ''' We have made this private method, since the Public method will be called from
    ''    ''' outside and doesnt not need to pass the Start Tag Parameter
    ''    ''' Where as need the param for internal use where a class contains a property of type collection
    ''    ''' or of type DO, and since our naming in the XMl is based on property name for the DO type and 
    ''    ''' class name for the collection item type, we need to pass the Start tag, so that on encounterng
    ''    ''' the End tag for it, we can return
    ''    ''' </summary>
    ''    ''' <param name="xmlReader"></param>
    ''    ''' <param name="StartTag"></param>
    ''    ''' <returns></returns>
    ''    ''' <remarks></remarks>

    ''    Private Overloads Function XMLToDO(ByVal xmlReader As System.Xml.XmlTextReader, ByVal StartTag As String) As Hashtable
    ''        Dim sys() As System.Reflection.FieldInfo
    ''        Dim t As Type
    ''        Dim ObjSys As System.Reflection.FieldInfo
    ''        Dim PrevNode As String = ""
    ''        Dim HS As New Hashtable

    ''        'We need to add Property names to the hash, as we have property name in lower case
    ''        'so me.getproperty(name as string) doesnt work as it is case sensitive
    ''        'Get all the Properties name in a Hashlist
    ''        t = Me.GetType
    ''        sys = t.GetFields(BindingFlags.DeclaredOnly _
    ''                            Or BindingFlags.Instance _
    ''                            Or BindingFlags.NonPublic _
    ''                            Or BindingFlags.Public)


    ''        If sys.Length > 0 Then
    ''            For i As Integer = 0 To sys.Length - 1
    ''                HS.Add(sys(i).Name.ToLower, sys(i))
    ''            Next
    ''        End If

    ''        xmlReader.WhitespaceHandling = Xml.WhitespaceHandling.None

    ''        While xmlReader.Read
    ''            Select Case xmlReader.NodeType

    ''                Case XmlNodeType.Element
    ''                    ObjSys = t.GetField(xmlReader.Name, (BindingFlags.DeclaredOnly _
    ''                            Or BindingFlags.Instance _
    ''                            Or BindingFlags.NonPublic _
    ''                            Or BindingFlags.Public))

    ''                    Dim baseT As Type

    ''                    baseT = t.BaseType

    ''                    If ObjSys Is Nothing Then
    ''                        While baseT IsNot Nothing AndAlso ObjSys Is Nothing
    ''                            ObjSys = baseT.GetField(xmlReader.Name, (BindingFlags.DeclaredOnly _
    ''                                                           Or BindingFlags.Instance _
    ''                                                           Or BindingFlags.NonPublic _
    ''                                                           Or BindingFlags.Public))

    ''                            baseT = baseT.BaseType
    ''                        End While
    ''                    End If


    ''                    XMLToDONonPrimitiveTypes(ObjSys, xmlReader, HS)

    ''                Case XmlNodeType.Text
    ''                    If Not ObjSys Is Nothing Then
    ''                        MakeDOFromXML(ObjSys, xmlReader)
    ''                        RemoveFromHs(HS, ObjSys.Name)
    ''                    End If

    ''                Case XmlNodeType.EndElement
    ''                    'This is required so that, when a DO properties are set, it returns to tis calee
    ''                    'when the end tag for that DO is encountered
    ''                    'Otherwise the reader will keep on reading for properties for other DO
    ''                    If StartTag = xmlReader.Name Then Return HS


    ''            End Select
    ''        End While

    ''        Return HS
    ''    End Function


    ''    Private Sub XMLToDONonPrimitiveTypes(ByVal ObjSys As FieldInfo, _
    ''                                            ByVal xmlReader As XmlTextReader, ByVal HS As Hashtable)
    ''        Dim ty1 As Type
    ''        Dim objDO As BDOBase

    ''        If ObjSys IsNot Nothing Then
    ''            ty1 = ObjSys.FieldType.GetInterface(GetType(MSPL.DO.IDataObject).Name, True)
    ''            If ty1 IsNot Nothing Then
    ''                objDO = Activator.CreateInstance(ObjSys.FieldType)
    ''                objDO.XMLToDO(xmlReader, ObjSys.Name)
    ''                ObjSys.SetValue(Me, objDO)
    ''                RemoveFromHs(HS, ObjSys.Name)
    ''            Else
    ''                Dim arr() As Attribute = ObjSys.FieldType.GetCustomAttributes(GetType(SerializableAttribute), True) 'GetInterface(GetType(ISerializable).Name, True)
    ''                If arr IsNot Nothing AndAlso Not CheckPrimitiveTypes(ObjSys) Then
    ''                    'Means that this is not an object of BDOBase (not ours),
    ''                    'so we simply Deserialize it as it was serialized the same way
    ''                    IsAlienObject = True
    ''                End If
    ''            End If
    ''        Else
    ''            'Do nothing as the Collection having this Object as property we Deserialize it
    ''        End If
    ''    End Sub


    ''    'http://msdn.microsoft.com/en-us/library/47zceaw7(VS.80).aspx
    ''    Private Sub MakeDOFromXML(ByVal ObjSys As FieldInfo, ByVal reader As XmlReader)
    ''        Dim formatter As BinaryFormatter
    ''        Dim obj As Object
    ''        Dim stream As MemoryStream
    ''        Dim strArr1() As String

    ''        Select Case ObjSys.FieldType.Name
    ''            Case GetType(String).Name '"String"
    ''                ObjSys.SetValue(Me, reader.Value.ToString)

    ''            Case GetType(Int32).Name '"Int32"
    ''                ObjSys.SetValue(Me, CInteger(reader.Value))

    ''            Case GetType(Boolean).Name '"Boolean"
    ''                ObjSys.SetValue(Me, ObjSQLConverter.DBToBoolean(reader.Value))

    ''            Case GetType(DateTime).Name '"DateTime"
    ''                ObjSys.SetValue(Me, ObjSQLConverter.DBToDateTime(reader.Value))

    ''            Case GetType(Color).Name '"Color"
    ''                ObjSys.SetValue(Me, Color.FromArgb(reader.Value))

    ''            Case GetType(Decimal).Name '"Decimal"
    ''                ObjSys.SetValue(Me, ObjSQLConverter.DBToDecimal(reader.Value))

    ''            Case GetType(Double).Name
    ''                ObjSys.SetValue(Me, ObjSQLConverter.DBToDouble(reader.Value))

    ''            Case GetType(Int64).Name
    ''                ObjSys.SetValue(Me, ObjSQLConverter.DBToInteger(reader.Value))

    ''            Case GetType(Byte).Name
    ''                ObjSys.SetValue(Me, ObjSQLConverter.DBToByte(reader.Value))

    ''            Case GetType(Char).Name
    ''                ObjSys.SetValue(Me, ObjSQLConverter.DBToString(reader.Value))

    ''            Case GetType(SByte).Name
    ''                ObjSys.SetValue(Me, ObjSQLConverter.DBToByte(reader.Value))

    ''            Case GetType(Int16).Name
    ''                ObjSys.SetValue(Me, ObjSQLConverter.DBToInteger(reader.Value))

    ''            Case GetType(Single).Name
    ''                ObjSys.SetValue(Me, ObjSQLConverter.DBToSingle(reader.Value))

    ''            Case GetType(UInt32).Name
    ''                ObjSys.SetValue(Me, ObjSQLConverter.DBToInteger(reader.Value))

    ''            Case GetType(UInt64).Name
    ''                ObjSys.SetValue(Me, ObjSQLConverter.DBToInteger(reader.Value))

    ''            Case GetType(UInt16).Name
    ''                ObjSys.SetValue(Me, ObjSQLConverter.DBToInteger(reader.Value))


    ''            Case Else
    ''                If ObjSys.FieldType.IsEnum Then
    ''                    ObjSys.SetValue(Me, [Enum].ToObject(ObjSys.FieldType, CInteger(reader.Value)))

    ''                ElseIf ObjSys.FieldType.IsArray Then
    ''                    strArr1 = reader.Value.ToString.Split(",")
    ''                    ObjSys.SetValue(Me, strArr1)

    ''                Else
    ''                    If IsAlienObject Then
    ''                        formatter = New BinaryFormatter()
    ''                        stream = New MemoryStream(StringToByteArray(reader.Value))
    ''                        obj = formatter.Deserialize(stream)
    ''                        ObjSys.SetValue(Me, obj)
    ''                        IsAlienObject = False
    ''                    End If
    ''                End If
    ''        End Select
    ''    End Sub



    ''    Private Function CheckPrimitiveTypes(ByVal ObjSys As FieldInfo) As Boolean
    ''        Select Case ObjSys.FieldType.Name
    ''            Case GetType(String).Name '"String"
    ''                Return True
    ''            Case GetType(Int32).Name '"Int32"
    ''                Return True
    ''            Case GetType(Boolean).Name '"Boolean"
    ''                Return True
    ''            Case GetType(DateTime).Name '"DateTime"
    ''                Return True
    ''            Case GetType(Color).Name '"Color"
    ''                Return True
    ''            Case GetType(Decimal).Name '"Decimal"
    ''                Return True
    ''            Case GetType(Double).Name '"Double
    ''                Return True
    ''            Case GetType(Int64).Name
    ''                Return True
    ''            Case GetType(Byte).Name
    ''                Return True
    ''            Case GetType(Char).Name
    ''                Return True
    ''            Case GetType(SByte).Name
    ''                Return True
    ''            Case GetType(Int16).Name
    ''                Return True
    ''            Case GetType(Single).Name
    ''                Return True
    ''            Case GetType(UInt32).Name
    ''                Return True
    ''            Case GetType(UInt64).Name
    ''                Return True
    ''            Case GetType(UInt16).Name
    ''                Return True

    ''            Case Else
    ''                If ObjSys.FieldType.IsEnum Then Return True
    ''                Return False
    ''        End Select

    ''    End Function


    ''    ''' <summary>
    ''    ''' This method Removes from Hash the Proeprties whose values have not been
    ''    ''' filled. We can use this Hash for any of our Specific work.
    ''    ''' Currently it serves us in the Preferences for Filling Default values
    ''    ''' </summary>
    ''    ''' <param name="HS"></param>
    ''    ''' <param name="name"></param>
    ''    ''' <remarks></remarks>
    ''    Private Sub RemoveFromHs(ByVal HS As Hashtable, ByVal name As String)
    ''        If HS Is Nothing Then Exit Sub
    ''        If HS.ContainsKey(name) Then HS.Remove(name)
    ''    End Sub


    ''#End Region


    ''#Region "Clone"


    ''    ' <summary>
    ''    ' Clone the object, and returning a reference to a cloned object.
    ''    'Ref = http://www.codeproject.com/KB/cs/cloneimpl_class.aspx
    ''    ' </summary>
    ''    ' <returns>Reference to the new cloned 
    ''    ' object.</returns
    ''    Public Overridable Function Clone() As Object Implements System.ICloneable.Clone
    ''        'First we create an instance of this specific type.
    ''        Dim newObject As BDOBase
    ''        'Dim paramInfo() As ParameterInfo
    ''        'Dim args() As Object
    ''        Dim ConstrctrInfo As ConstructorInfo

    ''        Dim cinfo() As ConstructorInfo = Me.GetType.GetConstructors

    ''        ''check for the parameter less constructor here
    ''        For i As Integer = 0 To cinfo.Length - 1
    ''            If cinfo(i).GetParameters.Length = 0 Then
    ''                ConstrctrInfo = cinfo(i)
    ''                Exit For
    ''            End If
    ''        Next

    ''        'if we do not get a default constructor then we take the first one
    ''        If ConstrctrInfo Is Nothing Then
    ''            Throw New Exception("This class does not have a default constructor. So it can not be cloned.")
    ''        End If

    ''        If Me.GetType().IsAbstract Then Throw New Exception("Abstract classes cannot be cloned")

    ''        newObject = Activator.CreateInstance(Me.GetType())
    ''        CopyTo(newObject)
    ''        Return newObject
    ''    End Function



    ''    Public Overridable Sub CopyTo(ByVal newObject As BDOBase)
    ''        Dim destType As Type
    ''        Dim sourceType As Type

    ''        sourceType = Me.GetType
    ''        destType = newObject.GetType()

    ''        CheckBaseObjects(sourceType, destType, newObject)

    ''    End Sub


    ''    Public Overridable Sub CopyFromParent(ByVal newObject As BDOBase)
    ''        Dim destType As Type
    ''        Dim sourceType As Type

    ''        sourceType = Me.GetType
    ''        destType = newObject.GetType().BaseType

    ''        CheckBaseObjects(sourceType, destType, newObject)

    ''    End Sub


    ''    ''' <summary>
    ''    ''' This recursively checks if there is a Baseclass of the Object passed 
    ''    ''' and sends that for cloning. This is needed as we are doing cloning on the Fields
    ''    ''' and one of the Limitation with .net framework is it doesnt return BaseClass's Private fields
    ''    ''' </summary>
    ''    ''' <param name="sourceType"></param>
    ''    ''' <param name="destType"></param>
    ''    ''' <param name="newObject"></param>
    ''    ''' <remarks></remarks>
    ''    Private Sub CheckBaseObjects(ByVal sourceType As Type, ByVal destType As Type, ByVal newObject As BDOBase)
    ''        Dim SourceBaseType As Type
    ''        Dim DestinationBaseType As Type

    ''        CloneObject(sourceType, destType, newObject)

    ''        SourceBaseType = sourceType.BaseType
    ''        DestinationBaseType = destType.BaseType

    ''        If Not SourceBaseType Is Nothing Then
    ''            CheckBaseObjects(SourceBaseType, DestinationBaseType, newObject)
    ''        End If

    ''    End Sub


    ''    Private Sub CloneObject(ByVal sourceType As Type, ByVal destType As Type, ByVal newObject As BDOBase, Optional ByVal srcObj As BDOBase = Nothing)
    ''        Dim destinationField As FieldInfo
    ''        Dim ICloneType As Type
    ''        Dim IClone As ICloneable
    ''        Dim IsCloneable As Boolean
    ''        Dim sourceField As FieldInfo
    ''        Dim sourceObject As BDOBase

    ''        If srcObj Is Nothing Then
    ''            sourceObject = Me
    ''        Else
    ''            sourceObject = srcObj
    ''        End If

    ''        For Each sourceField In sourceType.GetFields(BindingFlags.DeclaredOnly _
    ''                                                                            Or BindingFlags.Instance _
    ''                                                                            Or BindingFlags.NonPublic _
    ''                                                                            Or BindingFlags.Public)
    ''            IsCloneable = True

    ''            destinationField = destType.GetField(sourceField.Name, BindingFlags.DeclaredOnly Or BindingFlags.Instance Or BindingFlags.NonPublic Or BindingFlags.Public)

    ''            'For the Properties we do not want to do a Clone for
    ''            'we need to set NonCloneableAttribute on them
    ''            For Each attrb As Attribute In sourceField.GetCustomAttributes(False)
    ''                If attrb.ToString = GetType(NonCloneableAttribute).FullName Then
    ''                    IsCloneable = False
    ''                    Exit For
    ''                End If
    ''            Next

    ''            'We query if the fiels support the ICloneable interface.
    ''            If IsCloneable Then
    ''                ICloneType = sourceField.FieldType.GetInterface(GetType(ICloneable).Name, True)
    ''            End If

    ''            ' if the prop is cloneable, we clone it
    ''            ' else we simply set the reference
    ''            If ICloneType IsNot Nothing Then
    ''                'Getting the ICloneable interface from the object
    ''                IClone = sourceField.GetValue(sourceObject)

    ''                If IClone IsNot Nothing Then
    ''                    'We use the clone method to set the new value to the field
    ''                    destinationField.SetValue(newObject, IClone.Clone())
    ''                Else
    ''                    destinationField.SetValue(newObject, Nothing)
    ''                End If
    ''            Else

    ''                'Checks for and Sets the IEnumerable type values
    ''                If Not CheckListCollection(sourceField, destinationField, newObject) Then
    ''                    'If the field doesn't support the ICloneable interface then just set it
    ''                    destinationField.SetValue(newObject, sourceField.GetValue(sourceObject))
    ''                End If

    ''            End If


    ''        Next

    ''    End Sub


    ''    Private Function CheckListCollection(ByVal sourceField As FieldInfo, _
    ''                                      ByVal destinationField As FieldInfo, _
    ''                                       ByVal newObject As BDOBase) As Boolean


    ''        If sourceField.FieldType.IsGenericType Then
    ''            Return SetGenericListCollection(sourceField, destinationField, newObject)
    ''        Else
    ''            Return SetListCollection(sourceField, destinationField, newObject)
    ''        End If

    ''    End Function


    ''    Private Function SetListCollection(ByVal sourceField As FieldInfo, _
    ''                                      ByVal destinationField As FieldInfo, _
    ''                                       ByVal newObject As BDOBase) As Boolean

    ''        Dim IListType As Type
    ''        Dim ICollType As Type
    ''        Dim destList, sourceList As IList
    ''        Dim destColl, sourceColl As ICollection
    ''        Dim ObjDest As BDOBase


    ''        'This version support the IList and the
    ''        'IDictionary interfaces to iterate on collections.
    ''        IListType = destinationField.FieldType.GetInterface(GetType(IList).Name, True)
    ''        ICollType = destinationField.FieldType.GetInterface(GetType(ICollection).Name, True)

    ''        If IListType IsNot Nothing Then
    ''            'Getting the IList interface
    ''            sourceList = sourceField.GetValue(Me)
    ''            If sourceList Is Nothing Then Return False

    ''            destList = destinationField.GetValue(newObject)
    ''            If destList Is Nothing Then destList = Activator.CreateInstance(sourceField.GetType)

    ''            'so that any previous object in it gets cleared
    ''            destList.Clear()

    ''            For intI As Integer = 0 To sourceList.Count - 1
    ''                ObjDest = Activator.CreateInstance(sourceList(intI).GetType)
    ''                CloneObject(sourceList(intI).GetType, sourceList(intI).GetType, ObjDest, sourceList(intI))
    ''                destList.Add(ObjDest)
    ''            Next

    ''        ElseIf ICollType IsNot Nothing Then
    ''            sourceColl = sourceField.GetValue(newObject)
    ''            If sourceColl Is Nothing Then Return False

    ''            destinationField.SetValue(newObject, sourceColl)
    ''        Else
    ''            Return False
    ''        End If

    ''        Return True

    ''    End Function


    ''    Private Function SetGenericListCollection(ByVal sourceField As FieldInfo, _
    ''                                      ByVal destinationField As FieldInfo, _
    ''                                       ByVal newObject As BDOBase) As Boolean

    ''        Dim IListType As Type
    ''        Dim ICollType As Type
    ''        Dim destList, sourceList As IList
    ''        Dim destColl, sourceColl As ICollection
    ''        Dim genericType As Type
    ''        Dim constructed As Type
    ''        Dim genericArgs() As Type
    ''        Dim ObjDest As BDOBase

    ''        'This version support the IList and the
    ''        'IDictionary interfaces to iterate on collections.
    ''        IListType = destinationField.FieldType.GetInterface(GetType(IList).Name, True)
    ''        ICollType = destinationField.FieldType.GetInterface(GetType(ICollection).Name, True)

    ''        If IListType IsNot Nothing Then
    ''            'Getting the IList interface
    ''            sourceList = sourceField.GetValue(newObject)
    ''            If sourceList Is Nothing Then Return False

    ''            destList = destinationField.GetValue(newObject)
    ''            If destList Is Nothing Then

    ''                genericType = sourceField.FieldType.GetGenericTypeDefinition
    ''                genericArgs = sourceField.FieldType.GetGenericArguments

    ''                constructed = genericType.MakeGenericType(genericArgs)
    ''                destList = Activator.CreateInstance(constructed)
    ''            End If

    ''            'so that any previous object in it gets cleared
    ''            destList.Clear()
    ''            For intI As Integer = 0 To sourceList.Count - 1
    ''                ObjDest = Activator.CreateInstance(sourceList(intI).GetType)
    ''                CloneObject(sourceList(intI).GetType, sourceList(intI).GetType, ObjDest, sourceList(intI))
    ''                destList.Add(ObjDest)
    ''            Next


    ''        ElseIf ICollType IsNot Nothing Then
    ''            sourceColl = sourceField.GetValue(newObject)
    ''            If sourceColl Is Nothing Then Return False

    ''            destColl = destinationField.GetValue(newObject)
    ''            If destColl Is Nothing Then destColl = Activator.CreateInstance(sourceField.GetType)

    ''            For Each obj As Object In sourceColl
    ''                CloneObject(sourceField.GetType, destinationField.GetType, obj)
    ''            Next
    ''        Else
    ''            Return False
    ''        End If

    ''        Return True

    ''    End Function


    ''#End Region


#End Region




End Class
 
